Ext.define('DSS.addon.HLEEdit.GridPanel', {

    extend : 'Ext.grid.Panel',

    config : {
        enableColumnHide : false,
        enableColumnMove : false,
        selType : 'rowmodel',
        viewConfig : {
            forceFit : true
        }
    },

    store : null,
    editorWindow : null,
    curSelectedEvent : null,
    curSelectedEventId : null,
    btnNew : null,
    btnEdit : null,
    btnDelete : null,
    btnDisable : null,
    btnEnable : null,
    btnTest : null,
    defaultSortCol:'name',
    defaultSortDirection:'ASC',

    property : 'DSS.json.Property',
    editSelectedRecord: function ()  {
        var oParams = {};
        for (var iIndex=0;iIndex<this.curSelectedEvent.fields.items.length;iIndex++) {
            var sKey=this.curSelectedEvent.fields.items[iIndex].name
            oParams[sKey] = this.curSelectedEvent.get(sKey);
        }
        this.editorWindow.show();
        this.editorWindow.setValues(oParams);
    },
    
    constructor : function(config) {

        var me = this;

        this.initConfig(config);
        this.property = Ext.create(this.property, config);

        this.btnNew = new Ext.create('Ext.button.Button', {
            text : _('Neue Handlung'),
            iconCls : 'icon-add-action',
            handler : function() {
                //me.curSelectedEvent = null;
                //me.curSelectedEventId = null;
                me.getSelectionModel().deselectAll();
                me.editorWindow.setValues(null);
                me.editorWindow.show();
            }
        });
        this.btnEdit = new Ext.create('Ext.button.Button', {
            text : _('Handlung bearbeiten'),
            iconCls : 'icon-edit-action',
            disabled : true,
            handler : function() {
                me.editSelectedRecord();
            }
        });
        this.btnDelete = new Ext.create('Ext.button.Button', {
            text : _('Handlung l&ouml;schen'),
            iconCls : 'icon-delete-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('user-defined-actions-config', {
                    actions : 'delete',
                    id : me.curSelectedEvent.get('id')
                })
                me.waitAndReload();
            }
        });
        this.btnDisable = new Ext.create('Ext.button.Button', {
            text : _('Handlung deaktivieren'),
            iconCls : 'icon-disable-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('user-defined-actions-config', {
                    actions : 'disable',
                    id : me.curSelectedEvent.get('id')
                })
                me.waitAndReload();
            }
        });
        this.btnEnable = new Ext.create('Ext.button.Button', {
            text : _('Handlung aktivieren'),
            iconCls : 'icon-enable-action',
            hidden : true,
            disabled : true,
            handler : function() {
                dss.raiseEvent('user-defined-actions-config', {
                    actions : 'enable',
                    id : me.curSelectedEvent.get('id')
                })
                me.waitAndReload();
            }
        });
        this.btnTest = new Ext.create('Ext.button.Button', {
            text : _('Handlung testen'),
            iconCls : 'icon-test-action',
            disabled : true,
            handler : function() {
                dss.raiseEvent('highlevelevent', {
                    id : me.curSelectedEvent.get('id')
                });

                window.setTimeout(function() {
                    me.loadParameter(me);
                }, 2 * 1000);
            }

        });

        Ext.apply(this, {
            tbar : {
                items : [ this.btnNew, this.btnEdit, this.btnDisable, this.btnEnable, this.btnTest, '->', this.btnDelete ]
            }
        });

        this.callParent(arguments);
    },
    waitAndReload : function() {
        var me = this;
        var iSubscriptID = Math.floor(Math.random() * 100000000000);
        dss.ajaxSyncRequest('/json/event/subscribe', {
            subscriptionID : iSubscriptID,
            name : 'user-defined-actions-saved'
        });
        dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
            subscriptionID : iSubscriptID,
            timeout : 60 * 1000
        }, function() {
            me.loadParameter();
            dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
                subscriptionID : iSubscriptID,
                name : 'user-defined-actions-saved'
            });
        });
    },

    loadParameter : function(me) {
        if (me == null)
            me = this;
        dss.staticDataModel.initHLECache();
        var sString = dss.ajaxSyncRequest('/json/property/query', {
            'query' : '/usr/events/*(*)/*(*)/*(*)'
        });
        var result = Ext.JSON.decode(sString);
        
        var oResultObject=dss.parsePropertyQueryAnswer(result,[{name:'name',type:'simple'},
                                                               {name:'id',type:'simple'},
                                                               {name:'disabled',type:'condition/enabled'},
                                                               {name:'lastSaved',type:'simple'},
                                                               {name:'lastExecuted',type:'simple'},
                                                               {name:'actions',type:'action'}]);
        if (oResultObject!=null) {
            me.store.removeAll();
            me.store.add(oResultObject);
            me.store.sort(me.defaultSortCol,me.defaultSortDirection);
            if (me.curSelectedEventId != null) {
                var oToSelect = me.store.findRecord('id', me.curSelectedEventId);
                if (oToSelect != null) {
                    me.getSelectionModel().select(oToSelect);
                    me.curSelectedEvent = oToSelect;
                }
            }
        }
    },
    initComponent : function() {
        var me = this;

        me.store = Ext.create('Ext.data.Store', {
            storeId : 'hleStore',
            fields : [ {name:'id'}, 
                       {name:'disabled', sortType:'asBoolean'}, 
                       {name:'name', sortType:'asUCText'}, 
                       {name:'actions', sortType:'asAction'}, 
                       {name:'lastSaved', sortType:'asTimestamp'}, 
                       {name:'lastExecuted', sortType:'asTimestamp'} ],
            proxy : {
                type : 'memory',
                reader : {
                    type : 'json',
                    root : 'items'
                }
            }
        });

        var editorConfig = me.config;
        editorConfig.title = 'Benutzerdefinierte Handlung bearbeiten';
        editorConfig.staticControls = [ {
            type : 'text',
            label : _('Name'),
            id : 'name'
        }, {
            type : 'hidden',
            label : _('id'),
            id : 'id'
        } ];
        editorConfig.tabControls = [ {
            tabLabel : 'Auszuf&uuml;hrende Aktivit&auml;ten',
            tabDescription : 'Geben Sie die Aktivit&auml;ten ein, die in dieser benutzerdefinierten Handlung ausgef&uuml;hrt werden sollen.',
            content : [ {
                type : 'ActionConfigurator',
                id : 'actions',
                flex : true,
                noHighlevel:true
            } ]
        }, {
            tabLabel : 'Bedingungen',
            content : []
        } ];
        me.editorWindow = Ext.create('DSS.addon.HLEEditor', editorConfig);
        me.on({
            selectionchange : function(view, selections, options) {
                var me = this;
                if (selections.length > 0) {
                    me.curSelectedEvent = selections[0];
                    me.curSelectedEventId = selections[0].get('id');
                    me.btnEdit.setDisabled(false);
                    me.btnDelete.setDisabled(false);
                    me.btnDisable.setDisabled(me.curSelectedEvent.get('disabled'));
                    me.btnDisable.setVisible(!me.curSelectedEvent.get('disabled'));
                    me.btnEnable.setDisabled(!me.curSelectedEvent.get('disabled'));
                    me.btnEnable.setVisible(me.curSelectedEvent.get('disabled'));
                    me.btnTest.setDisabled(me.curSelectedEvent.get('disabled'));
                } else {
                    me.curSelectedEvent = null;
                    me.btnEdit.setDisabled(true);
                    me.btnDelete.setDisabled(true);
                    me.btnDisable.setDisabled(true);
                    me.btnEnable.setDisabled(true);
                    me.btnEnable.setVisible(false);
                    me.btnTest.setDisabled(true);
                }
            },          
            itemdblclick: function() {
                var me = this;
                if (me.curSelectedEvent != null) {
                    me.editSelectedRecord();
                };
            },
            sortchange: function (header,col,direction)
            {
                var me = this;
                me.defaultSortCol=col.dataIndex;
                me.defaultSortDirection=direction;
            }
        });
        me.editorWindow.on({
            'save' : function(oValue) {
                dss.raiseEvent('user-defined-actions-config', {
                    actions : 'save',
                    value : Ext.JSON.encode(oValue)
                })
                me.waitAndReload();
            }
        })
        me.callParent(arguments);
        me.initPage();
        me.loadParameter();
    }, // initComponent

    initPage : function() {
    },

    viewConfig : {
        stripeRows : true
    },

    columns : [
            {
                id : 'col-status',
                header : _('Active'),
                dataIndex : 'disabled',
                xtype: 'disabledcolumn',                 
                resizable : false,
                width : 50,
                listeners: {                  
                    checkchange : function (column,oValue,dataIndex, checked) {
                        if (oValue.get('disabled')==false)
                        {
                            dss.raiseEvent('user-defined-actions-config', {
                                actions : 'enable',
                                id : oValue.get('id')                        
                            });  
                            this.up('grid').waitAndReload();
                        } else {
                            dss.raiseEvent('user-defined-actions-config', {
                                actions : 'disable',
                                id : oValue.get('id')
                            });
                            this.up('grid').waitAndReload();
                        }
                    }
                }
            },
            {
                id : 'col-name',
                header : _('Name'),
                dataIndex : 'name',
                xtype: 'textcolumn',
                width : 200
            },
            {
                id : 'col-actions',
                header : 'Auszuf&uuml;hrende Aktivit&auml;t',
                dataIndex : 'actions', 
                maintainFlex : true,
                flex : 5,
                xtype: 'actioncolumn'
            },
            {
                id : 'col-changed',
                header : _('Bearbeitet am'),
                dataIndex : 'lastSaved',
                xtype: 'timestampcolumn',
                width : 200
            },
            {
                id : 'col-executed',
                header : _('zuletzt ausgef&uuml;hrt'),
                dataIndex : 'lastExecuted',
                xtype:'timestampcolumn',
                width : 200
            },

    ]
});

Ext.define('DSS.addon.HLEEditor', {
    extend : 'DSS.component.Editor'
});

function pad(c) {
    var i = parseInt(c, 10);
    return (i < 10 ? '0' + i : c.toString());
}
